#ifndef UAV_Imaging_configuration_h
#define UAV_Imaging_configuration_h

#define DEFAULT -1
#define ENABLED 1
#define DISABLED 0
#define LEAST_RECENT 0
#define MOST_RECENT 1
#define WAIT_ON_FULL 0
#define DROP_ON_FULL 1
//******************** Thread/Buffer configurations ********************
//Producer/Consumer semaphore buffer size
#define CAPTURED_IMAGE_BUFFER_SIZE      100
#define CAPTURED_IMAGE_BUFFER_FULL      WAIT_ON_FULL      
#define PROCESSED_IMAGE_BUFFER_SIZE     100
#define PROCESSED_IMAGE_BUFFER_FULL     WAIT_ON_FULL
//Number of image processing threads. When it is set to DEFAULT, number of threads is equal to number of processers.
#define IMAGE_PROCESSING_THREAD_COUNT   DEFAULT
//*********************************************************************


//************************ Target Information ************************
#define TARGET_MAX_DIMENSION_METER 2.4384     //8 feet
#define TARGET_MIN_DIMENSION_METER 0.6096     //2 feet
//********************************************************************

//******************* Image capturing configurations *******************
//portion of frame overlaps previous frame
#define OVERLAP_RATIO       0.1
#define CAPTURE_CONDITION_CHECKING_PERIOD 1     //in seconds, supporting decimal number
//**********************************************************************


//************************* File configurations *************************
//use /dev/shm for ramdisk
#define FILE_PATH_LENGTH        64
#define MAX_FILE_INPUT_LINE_SIZE     32
//captured image to be processed
#define FILE_PATH_CAPTURED      "/Volumes/Ramdisk/Image"
#define CAPTURED_IMAGE_FORMAT    "pgm"
#define DELETE_IMAGE_AFTER_PROCESSING     DISABLED
//processed image to be transmitted
#define FILE_PATH_PROCESSED     "/Volumes/Ramdisk/Image/Processed"
#define PROCESSED_IMAGE_FORMAT  "jpg"
#define DELETE_IMAGE_AFTER_TRANSMISSION   ENABLED

//image information text file
#define FILE_PATH_INFO          "/Volumes/Ramdisk/Image/Info"
#define INFO_FORMAT             "txt"
#define DELETE_INFO_AFTER_TRANSMISSION    ENABLED

#define LOCALPATH_COMMAND  "/Volumes/Ramdisk/Image/Command"

#define MIN_COMPRESSION_QUALITY_FOR_TARGET_IMAGE 40
#define MIN_COMPRESSION_QUALITY_FOR_OTHER_IMAGE 30
#define MAX_COMPRESSION_QUALITY 70
#define COMPRESSION_QUALITY_CHANGE_FOR_CAPTURING 2
#define COMPRESSION_QUALITY_CHANGE_FOR_TRANSMISSION 1

//*********************************************************************

//***************** Command Transmission configurations *****************
#define COMMAND_FILENAME        "command.txt"
#define COMMAND_LOADING_FREQUENCY    10    //in seconds
//*********************************************************************

//******************** Transmission configurations ********************
#define URL_LENGTH          64
#define GROUND_STATION_URL_IMAGE     "ftp://Shen:yjbqyb@127.0.0.1:21"
#define GROUND_STATION_URL_INFO       "ftp://Shen:yjbqyb@127.0.0.1:21"
#define GROUND_STATION_URL_COMMAND     "ftp://Shen:yjbqyb@127.0.0.1:21"
#define CONNECTION_TIMEOUT      1  //in seconds
#define DELAY_BEFORE_REATTEMPT 1     //in seconds, supporting decimal number
#define PROCESSING_AND_TRANSMISSION_PRIORITY MOST_RECENT
#define PRIORITIZE_TARGET_TRSNSMISSION      ENABLED
// **********************************************************************

//*********************** Camera configurations ***********************
#define CAMERA_WIDTH    1296
#define CAMERA_HEIGHT   964
//The dimension with same orientation as plane moves
#define CAMERA_MOUNTING_ORIENTATION_PIXEL     CAMERA_WIDTH
#define SHUTTER_SPEED   300             //Range: 0-1077
#define CAMERA_FRAME_BUFFER_SIZE 4      //internal buffer
//*********************************************************************

//******************** Image processing parameters ********************
#define DELTA_E_K_L             2
#define DELTA_E_K_1             0.048
#define DELTA_E_K_2             0.014

#define DELTA_E_EDGE_GRID_THRESHOLD 8
#define DELTA_E_FALSE_EDGE_THRESHOLD DELTA_E_EDGE_GRID_THRESHOLD
#define DELTA_E_CONNECTED_EDGE_THRESHOLD DELTA_E_EDGE_GRID_THRESHOLD
#define IMAGE_PROCESSING_GRID_SIZE      30          //must be even
#define MIN_GRID_COUNT 8
#define MAX_X_Y_RATIO 2

#define SIZE_ERROR_ALLOWANCE 1.5

#define SHAPE_RECOGNITION_STRICTNESS 0
#define APPROX_POLY_DOUBLE_PARAMETER 2
#define APPROX_POLY_INT_PARAMETER 1
//**********************************************************************

//************************ Miscellaneous ************************

#define DRAW_RECT_AROUND_TARGET 1
#define RECT_COLOR_R    255
#define RECT_COLOR_G    0
#define RECT_COLOR_B    0
#define RECT_THICKNESS  2

//*************************************************************


//************************ Program use ************************
#define COMMAND_LINE_LENGTH 32
#define DELAY_BEFORE_TERMINATION 2   //in seconds
//*************************************************************

// *************************** Debug ***************************
#define FPS_REPORT_FREQUENCY 2  // in seconds
//
#endif
