//
//  ftpTransmission.c
//  UAV Imaging
//
//  Created by Shen Yue on 12-03-29.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#include <stdio.h>
#include "ftpTransmission.h"

size_t getcontentlengthfunc(void *ptr, size_t size, size_t nmemb, void *stream);
size_t discardfunc(void *ptr, size_t size, size_t nmemb, void *stream);
size_t readfunc(void *ptr, size_t size, size_t nmemb, void *stream);
size_t writefunc(void *ptr, size_t size, size_t nmemb, void *stream);

CURL *initHandle(void)
{
    CURL *curlhandle=curl_easy_init();
    return curlhandle;
}

void initTransmission(void)
{
    curl_global_init(CURL_GLOBAL_ALL);
}

void cleanupHandle(CURL *curlhandle)
{
    curl_easy_cleanup(curlhandle);
}

void cleanupTransmission(void)
{
    curl_global_cleanup();
}

int upload(CURL *curlhandle, const char * remotepath, const char * localpath, long timeout)
{
    
    FILE *f;
    
    long uploaded_len = 0;
    
    CURLcode r = CURLE_GOT_NOTHING;
    
    f = fopen(localpath, "rb");
    
    if (f == NULL) {
        
        perror(NULL);
        
        return 0;
        
    }
    
    curl_easy_setopt(curlhandle, CURLOPT_UPLOAD, 1L);
    
    curl_easy_setopt(curlhandle, CURLOPT_URL, remotepath);
    
    if (timeout)
        
        curl_easy_setopt(curlhandle, CURLOPT_FTP_RESPONSE_TIMEOUT, timeout);
    
    curl_easy_setopt(curlhandle, CURLOPT_HEADERFUNCTION, getcontentlengthfunc);
    
    curl_easy_setopt(curlhandle, CURLOPT_HEADERDATA, &uploaded_len);
    
    curl_easy_setopt(curlhandle, CURLOPT_WRITEFUNCTION, discardfunc);
    
    curl_easy_setopt(curlhandle, CURLOPT_READFUNCTION, readfunc);
    
    curl_easy_setopt(curlhandle, CURLOPT_READDATA, f);
    
    curl_easy_setopt(curlhandle, CURLOPT_FTPPORT, "-"); /* disable passive mode */
    
    curl_easy_setopt(curlhandle, CURLOPT_FTP_CREATE_MISSING_DIRS, 1L);
    
    //curl_easy_setopt(curlhandle, CURLOPT_VERBOSE, 1L);
    
    
    curl_easy_setopt(curlhandle, CURLOPT_APPEND, 0L);
    r = curl_easy_perform(curlhandle); 
    //printf("%d\n",r);
    
    fclose(f);
    
    if (r == CURLE_OK)
    {
        return 1;
    }
    else {
        //fprintf(stderr, "%s/n", curl_easy_strerror(r));
        return 0;
    }
}

size_t discardfunc(void *ptr, size_t size, size_t nmemb, void *stream) 
{
    return size * nmemb;
}


size_t getcontentlengthfunc(void *ptr, size_t size, size_t nmemb, void *stream) 
{
    
    int r;
    
    long len = 0;
    
    /* _snscanf() is Win32 specific */
    
    //r = _snscanf(ptr, size * nmemb, "Content-Length: %ld/n", &len);
    
    r = sscanf(ptr, "Content-Length: %ld/n", &len);
    
    if (r) /* Microsoft: we don't read the specs */
        
        *((long *) stream) = len;
    
    return size * nmemb;
    
}

size_t readfunc(void *ptr, size_t size, size_t nmemb, void *stream)

{
    
    FILE *f = stream;
    
    size_t n;
    
    if (ferror(f))
        
        return CURL_READFUNC_ABORT;
    
    n = fread(ptr, size, nmemb, f) * size;
    
    return n;
    
}

size_t writefunc(void *ptr, size_t size, size_t nmemb, void *stream) 
{ 
    return fwrite(ptr, size, nmemb, stream); 
} 


int download(CURL *curlhandle, const char * remotepath, const char * localpath, long timeout) 
{ 
    
    FILE *f; 
    long filesize =0 ; 
    CURLcode r = CURLE_GOT_NOTHING; 
        
    f = fopen(localpath, "wb"); 
    
    if (f == NULL) { 
        
        perror(NULL); 
        
        return 0; 
        
    } 
    
    curl_easy_setopt(curlhandle, CURLOPT_URL, remotepath); 
    
    curl_easy_setopt(curlhandle, CURLOPT_CONNECTTIMEOUT, timeout); 
        
    curl_easy_setopt(curlhandle, CURLOPT_HEADERFUNCTION, getcontentlengthfunc); 
    
    curl_easy_setopt(curlhandle, CURLOPT_HEADERDATA, &filesize); 

    curl_easy_setopt(curlhandle, CURLOPT_WRITEFUNCTION, writefunc); 
    
    curl_easy_setopt(curlhandle, CURLOPT_WRITEDATA, f); 
    
    curl_easy_setopt(curlhandle, CURLOPT_NOPROGRESS, 1L); 
    
    //curl_easy_setopt(curlhandle, CURLOPT_VERBOSE, 1L); 
    
    r = curl_easy_perform(curlhandle); 
    
    fclose(f); 
    
    if (r == CURLE_OK) 
        
        return 1; 
    
    else { 
        
        //fprintf(stderr, "%s/n", curl_easy_strerror(r)); 
        
        return 0; 
        
    } 
    
} 