//
//  imageCapturing.c
//  UAV Imaging
//
//  Created by Shen Yue on 12-02-27.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#include <stdio.h>
#include "imageCapturing.h"
#include "configuration.h"
#include <sys/time.h>
#include <unistd.h>
#include "autopilot.h"
#include "utility.h"
#include "camera.h"

void waitForNextCapture(void);
void waitForCaptureCondition(void);
int checkForCaptureCondition(void);
long int capturePeriod(void);
int captureImage(void);
void storeImageInformation(int imageNumber,float altitude);

int captureControl(void)
{
    waitForNextCapture();
    waitForCaptureCondition();
    return captureImage();
}

int captureImage(void)
{
    static int imageNumber=50;
    //static int imageNumber=0;
    imageNumber++;
    if (imageNumber>59) imageNumber=51; // for testing
    int altitude=getAltitude();
    //capture and store image
    char path[FILE_PATH_LENGTH];
    sprintf(path, "%s/%d.%s",FILE_PATH_CAPTURED,imageNumber,CAPTURED_IMAGE_FORMAT);
    storeImage(path);
    
    storeImageInformation(imageNumber,altitude);
    //printf("captured:%d\n",imageNumber);
    return imageNumber;
}

void storeImageInformation(int imageNumber,float altitude)
{
    char path[FILE_PATH_LENGTH];
    sprintf(path, "%s/%d.%s",FILE_PATH_INFO,imageNumber,INFO_FORMAT);
    FILE *fp = fopen (path, "w");
    fprintf (fp, "altitude: %f\n",altitude);
    fclose(fp);
}

//in micro second
long int capturePeriod(void)
{
    return CAMERA_MOUNTING_ORIENTATION_PIXEL/getPixelsPerMeter(getAltitude())*(1-OVERLAP_RATIO)/getSpeed()*1000000;
}

void waitForCaptureCondition(void)
{
    while (checkForCaptureCondition()==0) 
    {
        usleep((useconds_t)(CAPTURE_CONDITION_CHECKING_PERIOD*1000000));
    }
}

int checkForCaptureCondition(void)
{
    //check conditions such as altitude, location
    return 1;
}

void waitForNextCapture(void)
{
    static long int lastCapture=0;
    if (lastCapture==0)
    {
        lastCapture=timer(0);
        return;
    }
    long int timeSinceLastCapture=timer(lastCapture);
    if(timeSinceLastCapture<capturePeriod())
    {
        usleep((useconds_t)(capturePeriod()-timeSinceLastCapture));
    }
    lastCapture=timer(0);
}