//
//  recognition.c
//  UAV Imaging
//
//  Created by Shen Yue on 12-03-04.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#include <stdio.h>
#include <stdlib.h>
#include "recognition.h"
#include "utility.h"
#include "configuration.h"

#define SHAPE_COUNT 4
#define SHAPE_NAME_LENGTH 16
#define NOT_TESTED  -1

typedef struct shapeData {
    char name[SHAPE_NAME_LENGTH];
    float min_hu1;
    float max_hu1;
    float min_hu2;
    float max_hu2;
    float min_hu3;
    float max_hu3;
    int min_edge;
    int max_edge;
    float min_EdgeDifference;
    float max_EdgeDifference;
} SHAPE_DATA;

SHAPE_DATA* loadShapeData(void);
void checkHu(CvHuMoments huMoments,ARRAY_INT_QUEUE *possibleShapes, SHAPE_DATA *shapeData);
void checkEdge(CvSeq* contour,ARRAY_INT_QUEUE *possibleShapes, SHAPE_DATA *shapeData);

char* recognition(CvSeq* contour)
{
    static SHAPE_DATA *shapeData=NULL;
    if (shapeData==NULL)
    {
        shapeData=loadShapeData();
    }
    ARRAY_INT_QUEUE *possibleShapes=mallocArrayIntQueue(SHAPE_COUNT);
    
    int i;
    for (i=0; i<SHAPE_COUNT; i++) {
        addToArrayIntQueueTail(possibleShapes, i);
    }
        
    CvMoments moments;
    CvHuMoments huMoments;
    cvMoments(contour, &moments, 1);
    cvGetHuMoments(&moments, &huMoments);
    checkHu(huMoments, possibleShapes,shapeData);
    
    if (possibleShapes->count>SHAPE_RECOGNITION_STRICTNESS)
    {
        checkEdge(contour, possibleShapes, shapeData);
    }
    
    printf("Hu1:%f\nHu2:%f\nHu3:%f\n",huMoments.hu1,huMoments.hu2,huMoments.hu3);
    printf("count:%d\n",possibleShapes->count);
    printf("%s\n",shapeData[removeFromArrayIntQueueHead(possibleShapes)].name);
    IplImage* contourImage = cvCreateImage( cvSize(50,50), 8, 3 );
    cvSet(contourImage, cvScalar(0, 0, 0, 0), NULL);
    CvScalar color = CV_RGB( 255, 255,255 );
    cvDrawContours( contourImage, contour, color, color, -1, CV_FILLED, 8 ,cvPoint(0, 0));
    cvNamedWindow("window", CV_WINDOW_AUTOSIZE);
    cvShowImage("window", contourImage);
    cvWaitKey(0);
    cvDestroyWindow("window");
    cvReleaseImage(&contourImage);
    
    
    char* ret=NULL;
    if (possibleShapes->count>0) {
        ret=shapeData[removeFromArrayIntQueueHead(possibleShapes)].name;
    }
    
    freeArrayIntQueue(possibleShapes);
    
    return ret;
}

void checkHu(CvHuMoments huMoments,ARRAY_INT_QUEUE *possibleShapes, SHAPE_DATA *shapeData)
{
    int i;
    int possibleShapeCount=possibleShapes->count;
    for (i=0;i<possibleShapeCount;i++)
    {
        int shapeNumber=removeFromArrayIntQueueHead(possibleShapes);
        if (withinRangeFloat(huMoments.hu1, shapeData[shapeNumber].min_hu1, shapeData[shapeNumber].max_hu1) 
            && withinRangeFloat(huMoments.hu2, shapeData[shapeNumber].min_hu2, shapeData[shapeNumber].max_hu2)
            && withinRangeFloat(huMoments.hu3, shapeData[shapeNumber].min_hu3, shapeData[shapeNumber].max_hu3))
        {
            addToArrayIntQueueTail(possibleShapes, shapeNumber);
        }
    }
}

void checkEdge(CvSeq* contour,ARRAY_INT_QUEUE *possibleShapes, SHAPE_DATA *shapeData)
{
    int i;
    CvSeq* polyContour;
    CvMemStorage* polyStorage = cvCreateMemStorage(0);
    polyContour=cvApproxPoly(contour, sizeof(CvContour), polyStorage, CV_POLY_APPROX_DP, APPROX_POLY_DOUBLE_PARAMETER , APPROX_POLY_INT_PARAMETER);
    
    //printf("Edge:%d\n",polyContour->total);  //for test
    
    int possibleShapeCount=possibleShapes->count;
    for (i=0;i<possibleShapeCount;i++)
    {
        int shapeNumber=removeFromArrayIntQueueHead(possibleShapes);
        if (withinRangeInt(polyContour->total, shapeData[shapeNumber].min_edge, shapeData[shapeNumber].max_edge))
        {
            addToArrayIntQueueTail(possibleShapes, shapeNumber);
        }
    }
        
    possibleShapeCount=possibleShapes->count;
    if (possibleShapeCount>SHAPE_RECOGNITION_STRICTNESS)
    {
        float min=cvArcLength(polyContour, cvSlice(0,1), 0);
        float max=min;
        float length;
        for (i=1;i<polyContour->total;i++)
        {
            length=cvArcLength(polyContour, cvSlice(i, i+1),0);
            if (length<min) min=length;
            else if (length>max) max=length;
        }
        float ratio=max/min;
        for (i=0;i<possibleShapeCount;i++)
        {
            int shapeNumber=removeFromArrayIntQueueHead(possibleShapes);
            if (withinRangeFloat(ratio, shapeData[shapeNumber].min_EdgeDifference, shapeData[shapeNumber].max_EdgeDifference))
            {
                addToArrayIntQueueTail(possibleShapes, shapeNumber);
            }
        }
    }
    cvReleaseMemStorage(&polyStorage);
}

SHAPE_DATA* loadShapeData(void)
{
    SHAPE_DATA *shapeData;
    shapeData=malloc(SHAPE_COUNT*sizeof(SHAPE_DATA));
    
    //square
    sprintf(shapeData[0].name, "square");
    shapeData[0].min_hu1=0.160;
    shapeData[0].max_hu1=0.175;
    shapeData[0].min_hu2=0;
    shapeData[0].max_hu2=0.001;
    shapeData[0].min_hu3=0;
    shapeData[0].max_hu3=0.001;
    shapeData[0].min_edge=4;
    shapeData[0].max_edge=4;
    shapeData[0].min_EdgeDifference=1;
    shapeData[0].max_EdgeDifference=1.5;
    
    //circle
    sprintf(shapeData[1].name, "circle");
    shapeData[1].min_hu1=0.160;
    shapeData[1].max_hu1=0.175;
    shapeData[1].min_hu2=0;
    shapeData[1].max_hu2=0.001;
    shapeData[1].min_hu3=0;
    shapeData[1].max_hu3=0.001;
    shapeData[1].min_edge=5;
    shapeData[1].max_edge=8;
    shapeData[1].min_EdgeDifference=1;
    shapeData[1].max_EdgeDifference=1.5;
    
    //triangle
    sprintf(shapeData[2].name, "triangle");
    shapeData[2].min_hu1=0.160;
    shapeData[2].max_hu1=0.175;
    shapeData[2].min_hu2=0;
    shapeData[2].max_hu2=0.001;
    shapeData[2].min_hu3=0;
    shapeData[2].max_hu3=0.001;
    shapeData[2].min_edge=3;
    shapeData[2].max_edge=3;
    shapeData[2].min_EdgeDifference=1;
    shapeData[2].max_EdgeDifference=1.5;
    
    
    
    //other   
    sprintf(shapeData[3].name, "other");
    shapeData[3].min_hu1=0.160;
    shapeData[3].max_hu1=0.300;
    shapeData[3].min_hu2=0;
    shapeData[3].max_hu2=0.01;
    shapeData[3].min_hu3=0;
    shapeData[3].max_hu3=0.001;
    shapeData[3].min_edge=3;
    shapeData[3].max_edge=8;
    shapeData[3].min_EdgeDifference=1;
    shapeData[3].max_EdgeDifference=10;
    
    return shapeData;
}
