//
//  utility.c
//  UAV Imaging
//
//  Created by Shen Yue on 12-02-27.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#include <stdio.h>
#include "utility.h"
#include <stdlib.h>
#include <sys/time.h>

//******************** ArrayIntQueue ********************
ARRAY_INT_QUEUE* mallocArrayIntQueue(int size)
{
    ARRAY_INT_QUEUE *queue;
    queue=malloc(sizeof(ARRAY_INT_QUEUE));
    queue->size=size;
    queue->array=malloc(size*sizeof(int));
    queue->head=0;
    queue->tail=1;
    queue->count=0;
    return queue;
}

void freeArrayIntQueue(ARRAY_INT_QUEUE* queue)
{
    free(queue->array);
    free(queue);
}

void addToArrayIntQueueHead(ARRAY_INT_QUEUE* queue,int number)
{
    queue->array[queue->head]=number;
    if (queue->head>0)
    {
        queue->head--;
    }
    else
    {
        queue->head=queue->size-1;
    }
    queue->count++;
}

void addToArrayIntQueueTail(ARRAY_INT_QUEUE* queue,int number)
{
    queue->array[queue->tail]=number;
    if (queue->tail<queue->size-1)
    {
        queue->tail++;
    }
    else
    {
        queue->tail=0;
    }
    queue->count++;
}

int removeFromArrayIntQueueHead(ARRAY_INT_QUEUE* queue)
{
    if (queue->head<queue->size-1)
    {
        queue->head++;
    }
    else
    {
        queue->head=0;
    }
    queue->count--;
    return queue->array[queue->head];
}

int removeFromArrayIntQueueTail(ARRAY_INT_QUEUE* queue)
{
    if (queue->tail>0)
    {
        queue->tail--;
    }
    else
    {
        queue->tail=queue->size-1;
    }
    queue->count--;
    return queue->array[queue->tail];
}
//************************************************************

//******************** twoDIntPointerArray ********************
TWO_D_INT_POINTER_ARRAY* mallocTwoDIntPointerArray(int width,int height)
{
    int i;
    TWO_D_INT_POINTER_ARRAY* array=malloc(sizeof(TWO_D_INT_POINTER_ARRAY));
    array->data=malloc(width*sizeof(int**));
    for (i=0;i<width;i++)
    {
        array->data[i]=malloc(height*sizeof(int*));
    }
    array->width=width;
    array->height=height;
    return array;
}

void freeTwoDIntPointerArray(TWO_D_INT_POINTER_ARRAY* array)
{
    int i;
    for (i=0;i<array->width;i++)
    {
        free(array->data[i]);
    }
    free(array->data);
    free(array);
}
//*************************************************************

//******************** intPointerQueue ********************
INT_POINTER_QUEUE *mallocIntPointerQueue(void)
{
    INT_POINTER_QUEUE *queue;
    queue=malloc(sizeof(INT_POINTER_QUEUE));
    queue->firstNode=NULL;
    queue->lastNode=NULL;
    return queue;
}

void addToIntPointerQueue(INT_POINTER_QUEUE *queue,int *value)
{
    INT_POINTER_SINGLY_LINKED_LIST_NODE *newNode=malloc(sizeof(INT_POINTER_SINGLY_LINKED_LIST_NODE));
    newNode->value=value;
    newNode->nextNode=NULL;
    if(queue->lastNode!=NULL)
        queue->lastNode->nextNode=newNode;
    else
        queue->firstNode=newNode;
    queue->lastNode=newNode;
}

int removeFromIntPointerQueue(INT_POINTER_QUEUE *queue,int **value)
{
    if (queue->firstNode!=NULL)
    {
        *value=queue->firstNode->value;
        
        if(queue->firstNode->nextNode!=NULL)
        {
            INT_POINTER_SINGLY_LINKED_LIST_NODE *newFirstNode=queue->firstNode->nextNode;
            free(queue->firstNode);
            queue->firstNode=newFirstNode;
        }
        else
        {
            free(queue->firstNode);
            queue->firstNode=NULL;
            queue->lastNode=NULL;
        }
        return 1;
    }
    else
        return 0;
}

void emptyIntPointerQueue(INT_POINTER_QUEUE *queue)
{
    INT_POINTER_SINGLY_LINKED_LIST_NODE *node=queue->firstNode;
    INT_POINTER_SINGLY_LINKED_LIST_NODE *nextNode;
    while (node!=NULL) 
    {
        nextNode=node->nextNode;
        free(node);
        node=nextNode;
    }
}

void freeIntPointerQueue(INT_POINTER_QUEUE *queue)
{
    emptyIntPointerQueue(queue);
    free(queue);
}
//**********************************************************

//******************** ArrayTwoIntQueue ********************
ARRAY_TWO_INT_QUEUE* mallocArrayTwoIntQueue(int size)
{
    ARRAY_TWO_INT_QUEUE *queue;
    queue=malloc(sizeof(ARRAY_TWO_INT_QUEUE));
    queue->size=size;
    queue->arrayX=malloc(size*sizeof(int));
    queue->arrayY=malloc(size*sizeof(int));
    queue->head=0;
    queue->tail=0;
    queue->count=0;
    return queue;
}

void freeArrayTwoIntQueue(ARRAY_TWO_INT_QUEUE* queue)
{
    free(queue->arrayX);
    free(queue->arrayY);
    free(queue);
}

void addToArrayTwoIntQueue(ARRAY_TWO_INT_QUEUE* queue,int x,int y)
{
    queue->arrayX[queue->tail]=x;
    queue->arrayY[queue->tail]=y;
    if (queue->tail<queue->size-1)
    {
        queue->tail++;
    }
    else
    {
        queue->tail=0;
    }
    queue->count++;
}

int removeFromArrayTwoIntQueue(ARRAY_TWO_INT_QUEUE* queue,int *x, int *y)
{
    if (queue->count>0)
    {
        *x=queue->arrayX[queue->head];
        *y=queue->arrayY[queue->head];
        if (queue->head<queue->size-1)
        {
            queue->head++;
        }
        else
        {
            queue->head=0;
        }
        queue->count--;
        return 1;
    }
    else
    {
        return 0;
    }
}
//************************************************************

// ************************* Timer *************************
long int timer(long int start)
{
    struct timeval currentTime;
    gettimeofday(&currentTime, NULL);
    // time in micro second
    return currentTime.tv_sec*1000000+currentTime.tv_usec-start;
}
//***********************************************************

//************************* withinRange *************************
int withinRangeInt(int value,int min,int max)
{
    return value>=min && value<=max;
}

int withinRangeFloat(float value,float min,float max)
{
    return value>=min && value<=max;
}
//***************************************************************
