----------------------------------------------------------------------------------
-- Module Name:    search_box - Behavioral 
-- Description: Find the potential marker by comparing standard deviation in a given frame
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity search_box is
	 generic(width : integer := 8; threshold : integer := 1024);
	 
    Port ( Clock : in  STD_LOGIC;
           Reset : in  STD_LOGIC;
           a : in  STD_LOGIC_VECTOR (width -1 downto 0);
           b : in  STD_LOGIC_VECTOR (width -1 downto 0);
           c : in  STD_LOGIC_VECTOR (width -1 downto 0);
           d : in  STD_LOGIC_VECTOR (width -1 downto 0);
           e : in  STD_LOGIC_VECTOR (width -1 downto 0);
           f : in  STD_LOGIC_VECTOR (width -1 downto 0);
           g : in  STD_LOGIC_VECTOR (width -1 downto 0);
           h : in  STD_LOGIC_VECTOR (width -1 downto 0);
           i : in  STD_LOGIC_VECTOR (width -1 downto 0);
           j : in  STD_LOGIC_VECTOR (width -1 downto 0);
           k : in  STD_LOGIC_VECTOR (width -1 downto 0);
           l : in  STD_LOGIC_VECTOR (width -1 downto 0);
           m : in  STD_LOGIC_VECTOR (width -1 downto 0);
           n : in  STD_LOGIC_VECTOR (width -1 downto 0);
           o : in  STD_LOGIC_VECTOR (width -1 downto 0);
           p : in  STD_LOGIC_VECTOR (width -1 downto 0);
	   
           Marker : out  STD_LOGIC_VECTOR (width - 1 downto 0));
end search_box;

architecture Behavioral of search_box is

signal sum : std_logic_vector (width-1 downto 0);
signal mean : std_logic_vector (width-1 downto 0);
signal sumq : std_logic_vector (2*width - 1 downto 0);

signal sumc : std_logic_vector (22 downto 0);
signal test1 : std_logic_vector (width-1 downto 0);
signal test2 : std_logic_vector (width-1 downto 0);
signal test3 : std_logic_vector (2*width-1 downto 0);
signal difference_a : std_logic_vector (width -1 downto 0);
signal difference_b : std_logic_vector (width -1 downto 0);
signal difference_c : std_logic_vector (width -1 downto 0);
signal difference_d : std_logic_vector (width -1 downto 0);
signal difference_e : std_logic_vector (width -1 downto 0);
signal difference_f : std_logic_vector (width -1 downto 0);
signal difference_g : std_logic_vector (width -1 downto 0);
signal difference_h : std_logic_vector (width -1 downto 0);
signal difference_i : std_logic_vector (width -1 downto 0);
signal difference_j : std_logic_vector (width -1 downto 0);
signal difference_k : std_logic_vector (width -1 downto 0);
signal difference_l : std_logic_vector (width -1 downto 0);
signal difference_m : std_logic_vector (width -1 downto 0);
signal difference_n : std_logic_vector (width -1 downto 0);
signal difference_o : std_logic_vector (width -1 downto 0);
signal difference_p : std_logic_vector (width -1 downto 0);

begin
		process(Clock)
		begin
			if rising_edge(Clock) then
				if Reset = '1' then
					sum <= (others => '0');
					mean <= (others => '0');
					sumq <= (others => '0');
					sumc <= (others => '0');
				else
					sum <= a+b+c+d+e+f+g+h+i+j+k+l+m+n+o+p;
					mean <= to_stdlogicvector(to_bitvector(a+b+c+d+e+f+g+h+i+j+k+l+m+n+o+p) sra 4);
					
					difference_a <= std_logic_vector(abs(signed(a-mean)));
					difference_b <= std_logic_vector(abs(signed(b-mean)));
					difference_c <= std_logic_vector(abs(signed(c-mean)));
					difference_d <= std_logic_vector(abs(signed(d-mean)));
					difference_e <= std_logic_vector(abs(signed(e-mean)));
					difference_f <= std_logic_vector(abs(signed(f-mean)));
					difference_g <= std_logic_vector(abs(signed(g-mean)));
					difference_h <= std_logic_vector(abs(signed(h-mean)));
					difference_i <= std_logic_vector(abs(signed(i-mean)));
					difference_j <= std_logic_vector(abs(signed(j-mean)));
					difference_k <= std_logic_vector(abs(signed(k-mean)));
					difference_l <= std_logic_vector(abs(signed(l-mean)));
					difference_m <= std_logic_vector(abs(signed(m-mean)));
					difference_n <= std_logic_vector(abs(signed(n-mean)));
					difference_o <= std_logic_vector(abs(signed(o-mean)));
					difference_p <= std_logic_vector(abs(signed(p-mean)));
					sumq <= difference_a*difference_a + difference_a*difference_a 
					+ difference_a*difference_a + difference_a*difference_a
					+ difference_a*difference_a + difference_a*difference_a
					+ difference_a*difference_a + difference_a*difference_a
					+ difference_a*difference_a + difference_a*difference_a
					+ difference_a*difference_a + difference_a*difference_a
					+ difference_a*difference_a + difference_a*difference_a
					+ difference_a*difference_a + difference_a*difference_a;
					
				end if;
			end if;
			if sumc < threshold then
				marker <= "11111111";
			else
				marker <= "00000000";
			end if;
		end process;

end Behavioral;
