----------------------------------------------------------
-- Top level of search box
----------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY search_box_top IS
  port (
        padClk, padReset : IN  STD_LOGIC;
        padA, padB, padC, padD, padE, padF, padG, padH, padI, padJ, padK, padL, padM, padN, padO, padP : IN  STD_LOGIC_VECTOR(7 downto 0);
	padMarker : OUT STD_LOGIC_VECTOR (7 downto 0)
	);
END search_box_top;

ARCHITECTURE Structure of search_box_top IS 

component search_box IS
  port (
        clock, Reset : IN  STD_LOGIC;
        A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P : IN  STD_LOGIC_VECTOR(7 downto 0);
	Marker : OUT STD_LOGIC_VECTOR(7 downto 0)
	);
END component;

component PDO12CDG 
  port(A     : in  std_logic;
       PAD   : out std_logic);
end component;

component PDUDGZ 
  port(Y     : out  std_logic;
       PAD   : in   std_logic);
end component;

signal  clock, Reset : STD_LOGIC;
signal  A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P : STD_LOGIC_VECTOR(7 downto 0);
signal	Marker : STD_LOGIC_VECTOR(7 downto 0);

--for all : PDUDGZ use entity Work.PDUDGZ(inpad_arc);
--for all : PDO12CDG use entity Work.PDO12CDG(outpad_arc);

begin

-- Input A pad mapping
  padAGen: for z in 7 downto 0 generate
    begin
        inpA: PDUDGZ port map (Y => A(z), PAD => padA(z));
    
  end generate;

-- Input B pad mapping
  padBGen: for z in 7 downto 0 generate
    begin
        inpB: PDUDGZ port map (Y => B(z), PAD => padB(z));
    
  end generate;

-- Input C pad mapping
  padCGen: for z in 7 downto 0 generate
    begin
        inpC: PDUDGZ port map (Y => C(z), PAD => padC(z));
    
  end generate;

-- Input D pad mapping
  padDGen: for z in 7 downto 0 generate
    begin
        inpD: PDUDGZ port map (Y => D(z), PAD => padD(z));
    
  end generate;

-- Input E pad mapping
  padEGen: for z in 7 downto 0 generate
    begin
        inpE: PDUDGZ port map (Y => E(z), PAD => padE(z));
    
  end generate;

-- Input F pad mapping
  padFGen: for z in 7 downto 0 generate
    begin
        inpF: PDUDGZ port map (Y => F(z), PAD => padF(z));
    
  end generate;

-- Input G pad mapping
  padGGen: for z in 7 downto 0 generate
    begin
        inpG: PDUDGZ port map (Y => G(z), PAD => padG(z));
    
  end generate;

-- Input H pad mapping
  padHGen: for z in 7 downto 0 generate
    begin
        inpH: PDUDGZ port map (Y => H(z), PAD => padH(z));
    
  end generate;

-- Input I pad mapping
  padIGen: for z in 7 downto 0 generate
    begin
        inpI: PDUDGZ port map (Y => I(z), PAD => padI(z));
    
  end generate;

-- Input J pad mapping
  padJGen: for z in 7 downto 0 generate
    begin
        inpJ: PDUDGZ port map (Y => J(z), PAD => padJ(z));
    
  end generate;

-- Input K pad mapping
  padKGen: for z in 7 downto 0 generate
    begin
        inpK: PDUDGZ port map (Y => K(z), PAD => padK(z));
    
  end generate;

-- Input L pad mapping
  padLGen: for z in 7 downto 0 generate
    begin
        inpL: PDUDGZ port map (Y => L(z), PAD => padL(z));
    
  end generate;

-- Input M pad mapping
  padMGen: for z in 7 downto 0 generate
    begin
        inpM: PDUDGZ port map (Y => M(z), PAD => padM(z));
    
  end generate;

-- Input N pad mapping
  padNGen: for z in 7 downto 0 generate
    begin
        inpN: PDUDGZ port map (Y => N(z), PAD => padN(z));
    
  end generate;

-- Input O pad mapping
  padOGen: for z in 7 downto 0 generate
    begin
        inpO: PDUDGZ port map (Y => O(z), PAD => padO(z));
    
  end generate;

-- Input P pad mapping
  padPGen: for z in 7 downto 0 generate
    begin
        inpP: PDUDGZ port map (Y => P(z), PAD => padP(z));
    
  end generate;  

-- Output marker pad mapping
  padMarkerGen: for z in 7 downto 0 generate
    begin
        opMarker: PDO12CDG port map (PAD => padMarker(z), A => Marker(z));
    
  end generate;

-- Misc pad mapping
  padClkG: 	PDUDGZ port map (Y => Clock, PAD => padClk);
  padResetG:	PDUDGZ port map (Y => Reset, PAD => padReset);

  coreG:        search_box port map (Clock => Clock, Reset => Reset,
                                    A => A, B => B, C => C, D => D, E=>E, F=>F,
				G=>G, H=>H, I=>I, J=>J, K=>K, L=>L, M=>M, N=>N,
				O=>O, P=>P,
				Marker => Marker);
END Structure;
